# ================================================================
# Calculate NCO increment value data for RevCounter Clock Project
# Picprojects
# 17/04/2019
# for python 3.6+
# ================================================================


# ================================================================
# Set these parameters to match settings on the instrument cluster
# Setting the largest wheel circumference and highest number of
# pulses per wheel revolution the instrument allows seems to give
# best results.
# ----------------------------------------------------------------
# Tyre circumference set on instrument cluster in Metres
circumference = 2.8
#
# Pulses per wheel revolution set on instrument cluster
pulse_per_rev = 12
#
# Adjust calculated speed to compensate for speedo error
error_adjust = 0.95
#
# Only apply error adjust for speeds above this
error_adjust_above = 10 
#
# ================================================================

speeddatafile = "speed-data.c" # output file name
MetresToMiles = 1609.344 # conversion constant
clock_period_uS = 25 # NCO input clock period in micro-seconds

NCO1_Clk_Freq = (10**6) / clock_period_uS  

print("NCO1 Clock Frequency = {}Hz".format(NCO1_Clk_Freq))

print('#include <stdint.h>', file=open(speeddatafile, 'w'))
print('// Lookup table with NCO1 increment data to drive speedometer', file=open(speeddatafile, 'a'))
print('// This file is generated by Python script "RevClockNcoMinData.py"', file=open(speeddatafile, 'a'))
print('const uint16_t SPEED[] = {', file=open(speeddatafile, 'a'))
print('{:4d}'.format(0), file=open(speeddatafile, 'a'), end='')

for speed in range (1,60):
    
    frequency = (pulse_per_rev * speed * MetresToMiles) / (circumference * 3600)

    if speed > error_adjust_above:
        nco_increment = ((frequency * 2**20)/ NCO1_Clk_Freq) * error_adjust
    else:
        nco_increment = (frequency * 2**20)/ NCO1_Clk_Freq

    int_nco_increment = round(nco_increment)
    
    print("Speed: {:2d}mph   Output Frequency: {:7.3f}Hz   NCO_Inc: {:4d}".format(speed,frequency, int_nco_increment))
      
    if speed % 10:
        print(', {:4d}'.format(int_nco_increment), file=open(speeddatafile, 'a'), end='')
    else:
        print(',\n{:4d}'.format(int_nco_increment), file=open(speeddatafile, 'a'), end='')

print('\n};',file=open(speeddatafile, 'a'), end='')

print("Data written to output file {}\n\n".format(speeddatafile))


